
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_UNSIGNED.all;


library UNISIM;
use UNISIM.vcomponents.all;


entity crio_arb is
  generic (
    SPI_DATA_LENGTH : integer := 72;
    SPI_CMD_LENGTH  : integer := 8
    );
  port (
    clk_i   : in std_logic;
    rst_n_i : in std_logic;

    crio_spi_clk_i  : in  std_logic;
    crio_spi_cs_n_i : in  std_logic;
    crio_spi_mosi_i : in  std_logic;
    crio_spi_miso_o : out std_logic;

    crio_id_select_n_i      : in  std_logic;
    crio_id_select_o        : out std_logic;
    crio_id_select_o_oe_n_o : out std_logic;

    crio_spi_func_i : in std_logic;

    crio_spi_eeprom_cs_n_o : out std_logic;
    crio_spi_data_cs_n_o   : out std_logic;

    crio_spi_eeprom_cs_led_n_o : out std_logic;
    crio_spi_data_cs_led_n_o   : out std_logic;

    wrpc_status_time_i : in std_logic_vector(SPI_DATA_LENGTH - 1 downto 0);

    data_reg_scratch1_o : out std_logic_vector(SPI_DATA_LENGTH - 1 downto 0);
    data_reg_scratch2_o : out std_logic_vector(SPI_DATA_LENGTH - 1 downto 0)

    );
end crio_arb;



architecture rtl of crio_arb is

  constant SPI_CMD_RD             : std_logic        := '1';
  constant SPI_CMD_WR             : std_logic        := '0';
  constant SPI_ADR_WRPC_STAT_TIME : std_logic_vector := "0000000";
  constant SPI_ADR_SCRATCH_1      : std_logic_vector := "0000010";
  constant SPI_ADR_SCRATCH_2      : std_logic_vector := "0000011";


  component crio_spi_slave
    generic (
      DATA_LENGTH : integer := 72;
      CMD_LENGTH  : integer := 8
      );
    port (
      clk_i   : in std_logic;
      rst_n_i : in std_logic;

      rx_cmd_o       : out std_logic_vector(CMD_LENGTH - 1 downto 0);
      rx_cmd_ready_o : out std_logic;
      rx_cmd_ovfl_o  : out std_logic;
      rx_cmd_rd_i    : in  std_logic;

      rx_data_o       : out std_logic_vector(DATA_LENGTH - 1 downto 0);
      rx_data_ready_o : out std_logic;
      rx_data_ovfl_o  : out std_logic;
      rx_data_rd_i    : in  std_logic;

      tx_data_i       : in  std_logic_vector(DATA_LENGTH - 1 downto 0);
      tx_data_ready_o : out std_logic;
      tx_data_wrerr_o : out std_logic;
      tx_data_wr_i    : in  std_logic;

      spi_clk_i  : in  std_logic;
      spi_cs_n_i : in  std_logic;
      spi_mosi_i : in  std_logic;
      spi_miso_o : out std_logic
      );
  end component;


  signal spi_rx_cmd       : std_logic_vector(SPI_CMD_LENGTH - 1 downto 0);
  signal spi_rx_cmd_ready : std_logic;
  signal spi_rx_cmd_ovfl  : std_logic;
  signal spi_rx_cmd_rd    : std_logic;

  signal spi_rx_data       : std_logic_vector(SPI_DATA_LENGTH - 1 downto 0);
  signal spi_rx_data_ready : std_logic;
  signal spi_rx_data_ovfl  : std_logic;
  signal spi_rx_data_rd    : std_logic;

  signal spi_tx_data       : std_logic_vector(SPI_DATA_LENGTH - 1 downto 0);
  signal spi_tx_data_ready : std_logic;
  signal spi_tx_data_wrerr : std_logic;
  signal spi_tx_data_wr    : std_logic;

  signal spi_eeprom_cs_n : std_logic;
  signal spi_data_cs_n   : std_logic;

  signal spi_eeprom_cs_n_reg1 : std_logic;
  signal spi_eeprom_cs_n_reg2 : std_logic;

  signal spi_data_cs_n_reg1 : std_logic;
  signal spi_data_cs_n_reg2 : std_logic;

  signal spi_rx_cmd_ready_reg  : std_logic;
  signal spi_rx_data_ready_reg : std_logic;

  signal spi_data_reg_scratch1 : std_logic_vector(SPI_DATA_LENGTH - 1 downto 0);
  signal spi_data_reg_scratch2 : std_logic_vector(SPI_DATA_LENGTH - 1 downto 0);

  signal crio_id_select_o_oe_n : std_logic;

  signal pulse_stretch_cnt_eeprom : std_logic_vector(23 downto 0);
  signal pulse_stretch_cnt_data   : std_logic_vector(23 downto 0);


begin

  -- SPI slave (CRIO data i/o normal mode)
  cmp_crio_spi_slave : crio_spi_slave
    generic map(
      DATA_LENGTH => SPI_DATA_LENGTH,
      CMD_LENGTH  => SPI_CMD_LENGTH
      )
    port map(
      clk_i   => clk_i,
      rst_n_i => rst_n_i,

      rx_cmd_o       => spi_rx_cmd,
      rx_cmd_ready_o => spi_rx_cmd_ready,
      rx_cmd_ovfl_o  => spi_rx_cmd_ovfl,
      rx_cmd_rd_i    => spi_rx_cmd_rd,

      rx_data_o       => spi_rx_data,
      rx_data_ready_o => spi_rx_data_ready,
      rx_data_ovfl_o  => spi_rx_data_ovfl,
      rx_data_rd_i    => spi_rx_data_rd,

      tx_data_i       => spi_tx_data,
      tx_data_ready_o => spi_tx_data_ready,
      tx_data_wrerr_o => spi_tx_data_wrerr,
      tx_data_wr_i    => spi_tx_data_wr,

      spi_clk_i  => crio_spi_clk_i,
      spi_cs_n_i => spi_data_cs_n,
      spi_mosi_i => crio_spi_mosi_i,
      spi_miso_o => crio_spi_miso_o
      );


  -- Chip select for SPI userdata (normal operation):
  spi_data_cs_n <= not crio_id_select_n_i or crio_spi_cs_n_i or not rst_n_i;


  -- Chip select output for external SPI EEPROM (ID mode):
  spi_eeprom_cs_n        <= crio_id_select_n_i or not crio_spi_func_i or crio_spi_cs_n_i or not rst_n_i;
  crio_spi_eeprom_cs_n_o <= spi_eeprom_cs_n;


  -- ID select output:
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      crio_id_select_o_oe_n <= '1';
    elsif rising_edge(clk_i) then
      crio_id_select_o_oe_n <= '0';
    end if;
  end process;

  cmp_idselect_obuft : OBUFT
    generic map (
      DRIVE      => 12,
      IOSTANDARD => "DEFAULT",
      SLEW       => "SLOW"
      )
    port map (
      O => crio_id_select_o,  -- Buffer output (connect directly to top-level port)
      I => '1',               -- Buffer input
      T => crio_id_select_o_oe_n  -- 3-state enable input (H : output = Z, L : output = input)
      );

  -- Pull-down for SPI-crio_id_select_o:
  PULLDN_inst : PULLDOWN
    port map (
      O => crio_id_select_o
      );

  crio_id_select_o_oe_n_o <= crio_id_select_o_oe_n;


  -- Write rx data to register 'n' depending on content of command register:

  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      spi_rx_cmd_ready_reg <= '0';
    elsif rising_edge(clk_i) then
      spi_rx_cmd_ready_reg <= spi_rx_cmd_ready;
    end if;
  end process;


  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      spi_rx_data_ready_reg <= '0';
    elsif rising_edge(clk_i) then
      spi_rx_data_ready_reg <= spi_rx_data_ready;
    end if;
  end process;

  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      spi_rx_cmd_rd <= '0';
    elsif rising_edge(clk_i) then
      if spi_rx_cmd_ready = '1' and spi_rx_cmd_ready_reg = '0' then
        spi_rx_cmd_rd <= '1';
      else
        spi_rx_cmd_rd <= '0';
      end if;
    end if;
  end process;

  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      spi_rx_data_rd <= '0';
    elsif rising_edge(clk_i) then
      if spi_rx_data_ready = '1' and spi_rx_data_ready_reg = '0' then
        spi_rx_data_rd <= '1';
      else
        spi_rx_data_rd <= '0';
      end if;
    end if;
  end process;

  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      spi_data_reg_scratch1 <= (others => '0');
      spi_data_reg_scratch2 <= (others => '0');
    elsif rising_edge(clk_i) then
      if spi_rx_data_ready = '1' and spi_rx_data_ready_reg = '0' then
        if spi_rx_cmd(SPI_CMD_LENGTH - 1) = SPI_CMD_WR then
          -- MSB of command = '0', i.e. master wants to write data to slave...
          case spi_rx_cmd(SPI_CMD_LENGTH - 2 downto 0) is
            when SPI_ADR_SCRATCH_1 =>
              spi_data_reg_scratch1 <= spi_rx_data;
            when SPI_ADR_SCRATCH_2 =>
              spi_data_reg_scratch2 <= spi_rx_data;
            when others =>
          -- Read only registers or undefined register address...
          end case;
        end if;
      end if;
    end if;
  end process;


  -- Setup tx data depending on content of command register:

  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      spi_tx_data_wr <= '0';
    elsif rising_edge(clk_i) then
      if spi_rx_cmd_ready = '1' and spi_rx_cmd_ready_reg = '0' then
        if spi_rx_cmd(SPI_CMD_LENGTH - 1) = '1' then
          -- MSB of command = '1', i.e. master wants to read data from slave...
          spi_tx_data_wr <= '1';
        else
          spi_tx_data_wr <= '0';
        end if;
      else
        spi_tx_data_wr <= '0';
      end if;
    end if;
  end process;


  TX_DATA : process (spi_rx_cmd, spi_data_reg_scratch1, spi_data_reg_scratch2, wrpc_status_time_i) is
  begin
    case spi_rx_cmd(SPI_CMD_LENGTH - 2 downto 0) is
      when SPI_ADR_WRPC_STAT_TIME =>
        spi_tx_data <= wrpc_status_time_i;
      when SPI_ADR_SCRATCH_1 =>
        spi_tx_data <= spi_data_reg_scratch1;
      when SPI_ADR_SCRATCH_2 =>
        spi_tx_data <= spi_data_reg_scratch2;
      when others =>
          -- Undefined register address...
        spi_tx_data <= (others => '1');
    end case;
  end process;


  -- Pulse stretchers for LEDs

  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      spi_data_cs_n_reg1 <= '1';
      spi_data_cs_n_reg2 <= '1';
    elsif rising_edge(clk_i) then
      spi_data_cs_n_reg1 <= spi_data_cs_n;
      spi_data_cs_n_reg2 <= spi_data_cs_n_reg1;
    end if;
  end process;

  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      pulse_stretch_cnt_data <= (others => '1');
    elsif rising_edge(clk_i) then
      if spi_data_cs_n_reg2 = '0' then
        pulse_stretch_cnt_data <= (others => '0');
      elsif pulse_stretch_cnt_data(23) = '0' then
        pulse_stretch_cnt_data <= pulse_stretch_cnt_data + 1;
      end if;
    end if;
  end process;

  crio_spi_data_cs_led_n_o <= pulse_stretch_cnt_data(23);


  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      spi_eeprom_cs_n_reg1 <= '1';
      spi_eeprom_cs_n_reg2 <= '1';
    elsif rising_edge(clk_i) then
      spi_eeprom_cs_n_reg1 <= spi_eeprom_cs_n;
      spi_eeprom_cs_n_reg2 <= spi_eeprom_cs_n_reg1;
    end if;
  end process;

  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      pulse_stretch_cnt_eeprom <= (others => '1');
    elsif rising_edge(clk_i) then
      if spi_eeprom_cs_n_reg2 = '0' then
        pulse_stretch_cnt_eeprom <= (others => '0');
      elsif pulse_stretch_cnt_eeprom(23) = '0' then
        pulse_stretch_cnt_eeprom <= pulse_stretch_cnt_eeprom + 1;
      end if;
    end if;
  end process;

  crio_spi_eeprom_cs_led_n_o <= pulse_stretch_cnt_eeprom(23);

end rtl;


