
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library UNISIM;
use UNISIM.vcomponents.all;


entity crio_spi_slave is
  generic (
    DATA_LENGTH : integer := 72;
    CMD_LENGTH  : integer := 8
    );
  port (
    clk_i   : in std_logic;
    rst_n_i : in std_logic;

    rx_cmd_o       : out std_logic_vector(CMD_LENGTH - 1 downto 0);
    rx_cmd_ready_o : out std_logic;
    rx_cmd_ovfl_o  : out std_logic;
    rx_cmd_rd_i    : in  std_logic;

    rx_data_o       : out std_logic_vector(DATA_LENGTH - 1 downto 0);
    rx_data_ready_o : out std_logic;
    rx_data_ovfl_o  : out std_logic;
    rx_data_rd_i    : in  std_logic;

    tx_data_i       : in  std_logic_vector(DATA_LENGTH - 1 downto 0);
    tx_data_ready_o : out std_logic;
    tx_data_wrerr_o : out std_logic;
    tx_data_wr_i    : in  std_logic;

    spi_clk_i  : in  std_logic;
    spi_cs_n_i : in  std_logic;
    spi_mosi_i : in  std_logic;
    spi_miso_o : out std_logic
    );
end;

architecture arch of crio_spi_slave is

  constant MOSI_EDGE : std_logic := '1';  -- MOSI data input is captured and shifted on this spi clock edge
  constant MISO_EDGE : std_logic := '0';  -- MISO data output is updated on this spi clock edge

  signal rx_bit_cnt        : std_logic_vector(6 downto 0);
  signal rx_shift_reg      : std_logic_vector(CMD_LENGTH + DATA_LENGTH - 1 downto 0);
  signal rx_cmd_done       : std_logic;
  signal rx_data_done      : std_logic;
  signal rx_cmd_done_reg1  : std_logic;
  signal rx_cmd_done_reg2  : std_logic;
  signal rx_data_done_reg1 : std_logic;
  signal rx_data_done_reg2 : std_logic;
  signal rx_data_done_reg3 : std_logic;
  signal rx_cmd_reg        : std_logic_vector(CMD_LENGTH - 1 downto 0);
  signal rx_data_reg       : std_logic_vector(DATA_LENGTH - 1 downto 0);
  signal rx_cmd_ready_reg  : std_logic;
  signal rx_data_ready_reg : std_logic;
  signal rx_cmd_ovfl_reg   : std_logic;
  signal rx_data_ovfl_reg  : std_logic;

  signal tx_bit_cnt        : std_logic_vector(6 downto 0);
  signal tx_data_reg       : std_logic_vector(DATA_LENGTH - 1 downto 0);
  signal tx_data_ready_reg : std_logic;
  signal tx_done           : std_logic;
  signal tx_done_reg1      : std_logic;
  signal tx_done_reg2      : std_logic;
  signal tx_done_reg3      : std_logic;
  signal tx_data_wrerr_reg : std_logic;

  signal tx_shift_reg : std_logic_vector(DATA_LENGTH - 1 downto 0);

begin

  ---------------------------------------------------------------------------------------------
  -- Receive data
  ---------------------------------------------------------------------------------------------

  -- Rx bit counter (spi clock domain):
  process(spi_clk_i, rst_n_i, spi_cs_n_i)
  begin
    if rst_n_i = '0' or spi_cs_n_i = '1' then
      rx_bit_cnt <= (others => '0');
    elsif spi_clk_i'event and spi_clk_i = MOSI_EDGE then
      rx_bit_cnt <= rx_bit_cnt + 1;
    end if;
  end process;


  -- Rx data shift register (spi clock domain):
  process(spi_clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_shift_reg <= (others => '0');
    elsif spi_clk_i'event and spi_clk_i = MOSI_EDGE then
      if spi_cs_n_i = '0' then
        rx_shift_reg <= rx_shift_reg(CMD_LENGTH + DATA_LENGTH - 2 downto 0) & spi_mosi_i;
      end if;
    end if;
  end process;


  -- Rx command done (spi clock domain):
  process(spi_clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_cmd_done <= '0';
    elsif spi_clk_i'event and spi_clk_i = MOSI_EDGE then
      if rx_bit_cnt = CMD_LENGTH - 1 then
        rx_cmd_done <= '1';
      else
        rx_cmd_done <= '0';
      end if;
    end if;
  end process;


  -- Rx data done (spi clock domain):
  process(spi_clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_data_done <= '0';
    elsif spi_clk_i'event and spi_clk_i = MOSI_EDGE then
      if rx_bit_cnt = CMD_LENGTH + DATA_LENGTH - 1 then
        rx_data_done <= '1';
      else
        rx_data_done <= '0';
      end if;
    end if;
  end process;


  -- Rx command done (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_cmd_done_reg1 <= '0';
      rx_cmd_done_reg2 <= '0';
    elsif rising_edge(clk_i) then
      rx_cmd_done_reg1 <= rx_cmd_done;
      rx_cmd_done_reg2 <= rx_cmd_done_reg1;
    end if;
  end process;


  -- Rx data done (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_data_done_reg1 <= '0';
      rx_data_done_reg2 <= '0';
      rx_data_done_reg3 <= '0';
    elsif rising_edge(clk_i) then
      rx_data_done_reg1 <= rx_data_done;
      rx_data_done_reg2 <= rx_data_done_reg1;
      rx_data_done_reg3 <= rx_data_done_reg2;
    end if;
  end process;


  -- Rx command output register (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_cmd_reg <= (others => '0');
    elsif rising_edge(clk_i) then
      if rx_cmd_done_reg1 = '1' and rx_cmd_done_reg2 = '0' then
        rx_cmd_reg <= rx_shift_reg(CMD_LENGTH - 1 downto 0);
      end if;
    end if;
  end process;

  rx_cmd_o <= rx_cmd_reg;


  -- Rx command ready flag (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_cmd_ready_reg <= '0';
    elsif rising_edge(clk_i) then
      if rx_cmd_done_reg1 = '1' and rx_cmd_done_reg2 = '0' then
        rx_cmd_ready_reg <= '1';
      elsif rx_cmd_rd_i = '1' then
        rx_cmd_ready_reg <= '0';
      end if;
    end if;
  end process;

  rx_cmd_ready_o <= rx_cmd_ready_reg;


  -- Rx command overflow flag (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_cmd_ovfl_reg <= '0';
    elsif rising_edge(clk_i) then
      if rx_cmd_done_reg1 = '1' and rx_cmd_done_reg2 = '0' and rx_cmd_ready_reg = '1' then
        rx_cmd_ovfl_reg <= '1';
      elsif rx_cmd_rd_i = '1' then
        rx_cmd_ovfl_reg <= '0';
      end if;
    end if;
  end process;

  rx_cmd_ovfl_o <= rx_cmd_ovfl_reg;


  -- Rx data output register (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_data_reg <= (others => '0');
    elsif rising_edge(clk_i) then
      if rx_data_done_reg1 = '1' and rx_data_done_reg2 = '0' then
        rx_data_reg <= rx_shift_reg(DATA_LENGTH - 1 downto 0);
      end if;
    end if;
  end process;

  rx_data_o <= rx_data_reg;


  -- Rx data ready flag (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_data_ready_reg <= '0';
    elsif rising_edge(clk_i) then
      if rx_data_done_reg2 = '1' and rx_data_done_reg3 = '0' then
        rx_data_ready_reg <= '1';
      elsif rx_data_rd_i = '1' then
        rx_data_ready_reg <= '0';
      end if;
    end if;
  end process;

  rx_data_ready_o <= rx_data_ready_reg;


  -- Rx data overflow flag (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      rx_data_ovfl_reg <= '0';
    elsif rising_edge(clk_i) then
      if rx_data_done_reg2 = '1' and rx_data_done_reg3 = '0' and rx_data_ready_reg = '1' then
        rx_data_ovfl_reg <= '1';
      elsif rx_cmd_rd_i = '1' then
        rx_data_ovfl_reg <= '0';
      end if;
    end if;
  end process;

  rx_data_ovfl_o <= rx_data_ovfl_reg;



  ---------------------------------------------------------------------------------------------
  -- Transmit data
  ---------------------------------------------------------------------------------------------

  -- Tx bit counter (spi clock domain):
  process(spi_clk_i, rst_n_i, spi_cs_n_i)
  begin
    if rst_n_i = '0' or spi_cs_n_i = '1' then
      tx_bit_cnt <= (others => '0');
    elsif spi_clk_i'event and spi_clk_i = MISO_EDGE then
      if rx_bit_cnt >= CMD_LENGTH then
        tx_bit_cnt <= tx_bit_cnt + 1;
      end if;
    end if;
  end process;


  -- Tx data serial output (spi clock domain):
  process(spi_clk_i, rst_n_i, spi_cs_n_i)
  begin
    if rst_n_i = '0' or spi_cs_n_i = '1' then
      tx_shift_reg <= (others => '0');
    elsif spi_clk_i'event and spi_clk_i = MISO_EDGE then
      if rx_bit_cnt = "0001000" then
        tx_shift_reg <= tx_data_reg;
      else
        tx_shift_reg <= tx_shift_reg(DATA_LENGTH - 2 downto 0) & "0";
      end if;
    end if;
  end process;

  spi_miso_o <= 'Z' when tx_bit_cnt = "0000000" or rx_cmd_reg(CMD_LENGTH - 1) = '0' else tx_shift_reg(DATA_LENGTH - 1);

  -- Pull-up for SPI-MISO:
  PULLUP_inst : PULLUP
    port map (
      O => spi_miso_o
      );


  -- Tx done (spi clock domain):
  process(spi_clk_i, rst_n_i, spi_cs_n_i)
  begin
    if rst_n_i = '0' or spi_cs_n_i = '1' then
      tx_done <= '0';
    elsif spi_clk_i'event and spi_clk_i = MOSI_EDGE then
      if tx_bit_cnt = DATA_LENGTH then
        tx_done <= '1';
      else
        tx_done <= '0';
      end if;
    end if;
  end process;


  -- Tx done (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      tx_done_reg1 <= '0';
      tx_done_reg2 <= '0';
      tx_done_reg3 <= '0';
    elsif rising_edge(clk_i) then
      tx_done_reg1 <= tx_done;
      tx_done_reg2 <= tx_done_reg1;
      tx_done_reg3 <= tx_done_reg2;
    end if;
  end process;


  -- Tx data ready (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      tx_data_ready_reg <= '1';
    elsif rising_edge(clk_i) then
      if tx_data_wr_i = '1' then
        tx_data_ready_reg <= '0';
      elsif tx_done_reg2 = '1' and tx_done_reg3 = '0' then
        tx_data_ready_reg <= '1';
      end if;
    end if;
  end process;

  tx_data_ready_o <= tx_data_ready_reg;


  -- Tx data write error (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      tx_data_wrerr_reg <= '0';
    elsif rising_edge(clk_i) then
      if tx_data_ready_reg = '0' and tx_data_wr_i = '1' then
        tx_data_wrerr_reg <= '1';
      elsif tx_data_wr_i = '1' then
        tx_data_wrerr_reg <= '0';
      end if;
    end if;
  end process;

  tx_data_wrerr_o <= tx_data_wrerr_reg;


  -- Tx data input register (fabric clock domain):
  process(clk_i, rst_n_i)
  begin
    if rst_n_i = '0' then
      tx_data_reg <= (others => '0');
    elsif rising_edge(clk_i) then
      if tx_data_ready_reg = '1' and tx_data_wr_i = '1' then
        tx_data_reg <= tx_data_i;
      end if;
    end if;
  end process;


end arch;
