-------------------------------------------------------------------------------
-- Title      : WRPC PMU design for CRIO-WR
-- Project    : CRIO-WR
-- URL        : http://www.ohwr.org/projects/crio-wr/wiki
-------------------------------------------------------------------------------
-- File       : crio_wr_pmu_top.vhd
-- Author(s)  : Dimitrios Lampridis <dimitrios.lampridis@cern.ch>
-- Company    : CERN (BE-CO-HT)
-- Created    : 2017-05-24
-- Standard   : VHDL'93
-------------------------------------------------------------------------------
-- Description: Top-level file for the WRPC PMU design on the CRIO-WR.
--
-- This is the top HDL that instanciates the WR PTP Core together with
-- its peripherals to be run on a CRIO-WR module. It also includes a 50Hz
-- PPS-aligned pulse output, to be used by the PMU.
--
-- Based on original work by Daniel Florin,
-- Physik Institut, Universitaet Zuerich, Switzerland
--
-------------------------------------------------------------------------------
-- Copyright (c) 2017 CERN
-------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--
-- This source file is free software; you can redistribute it
-- and/or modify it under the terms of the GNU Lesser General
-- Public License as published by the Free Software Foundation;
-- either version 2.1 of the License, or (at your option) any
-- later version.
--
-- This source is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied
-- warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-- PURPOSE.  See the GNU Lesser General Public License for more
-- details.
--
-- You should have received a copy of the GNU Lesser General
-- Public License along with this source; if not, download it
-- from http://www.gnu.org/licenses/lgpl-2.1.html
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.gencores_pkg.all;
use work.wrcore_pkg.all;
use work.wishbone_pkg.all;
use work.etherbone_pkg.all;
use work.wr_fabric_pkg.all;
use work.endpoint_pkg.all;
use work.streamers_pkg.all;
use work.wr_xilinx_pkg.all;
use work.wr_board_pkg.all;

library unisim;
use unisim.vcomponents.all;

entity crio_wr_pmu_top is
  generic(
    -- set to 1 to speed up some initialization processes during simulation
    g_simulation : integer := 0;
    -- Select whether to include external ref clock input
    g_with_external_clock_input : boolean := TRUE;
    -- setting g_dpram_initf to file path will result in syntesis/simulation using the
    -- content of this file to run LM32 microprocessor
    -- setting g_dpram_init to empty string (i.e."") will result in synthesis/simulation
    -- with empty RAM for the LM32 (it will not work until code is loaded)
    -- NOTE: the path is correct when used from the synthesis folder (this is where
    --       ISE calls the function to find the file, the path is not correct for where
    --       this file is stored, i.e. in the top/ folder)
    g_dpram_initf : string := "../../ip_cores/wr-cores/bin/wrpc/wrc_phy8.bram"
    );
  port (
    ---------------------------------------------------------------------------
    -- Clocks/resets
    ---------------------------------------------------------------------------

    -- Local oscillators
    clk_20m_vcxo_i      : in std_logic;  -- 20MHz VCXO clock
    clk_125m_pllref_p_i : in std_logic;  -- 125 MHz PLL reference
    clk_125m_pllref_n_i : in std_logic;
    clk_125m_gtp_n_i    : in std_logic;  -- 125 MHz GTP reference
    clk_125m_gtp_p_i    : in std_logic;

    ---------------------------------------------------------------------------
    -- SPI interface to DAC
    ---------------------------------------------------------------------------

    plldac_sclk_o   : out std_logic;  -- AD5663, dual DAC
    plldac_din_o    : out std_logic;
    plldac_clr_n_o  : out std_logic;
    plldac_ldac_n_o : out std_logic;
    plldac_sync_n_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SFP I/O for transceiver
    ---------------------------------------------------------------------------

    sfp_txp_o         : out   std_logic;
    sfp_txn_o         : out   std_logic;
    sfp_rxp_i         : in    std_logic;
    sfp_rxn_i         : in    std_logic;
    sfp_mod_def0_i    : in    std_logic;  -- sfp detect
    sfp_mod_def1_b    : inout std_logic;  -- scl
    sfp_mod_def2_b    : inout std_logic;  -- sda
    sfp_rate_select_o : out   std_logic;
    sfp_tx_fault_i    : in    std_logic;
    sfp_tx_disable_o  : out   std_logic;
    sfp_los_i         : in    std_logic;

    ---------------------------------------------------------------------------
    -- Onewire interface
    ---------------------------------------------------------------------------

    onewire_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- UART
    ---------------------------------------------------------------------------

    uart_rxd_i : in  std_logic;
    uart_txd_o : out std_logic;

    ---------------------------------------------------------------------------
    -- Flash memory SPI interface
    ---------------------------------------------------------------------------

    flash_sclk_o : out std_logic;
    flash_ncs_o  : out std_logic;
    flash_mosi_o : out std_logic;
    flash_miso_i : in  std_logic;

    ---------------------------------------------------------------------------
    -- I2C EEPROM interface
    ---------------------------------------------------------------------------

    eeprom_scl_b : inout std_logic;  -- 24AA64, 8k x 8
    eeprom_sda_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- LEDs buttons and User I/O
    ---------------------------------------------------------------------------

    -- General purpose interface
    --userio_1_p_b : out std_logic;  -- CON730_6  front panel (DSUB HD15)
    --userio_1_n_b : out std_logic;  -- CON730_1  front panel (DSUB HD15)
    --userio_2_p_b : out std_logic;  -- CON730_7  front panel (DSUB HD15)
    --userio_2_n_b : out std_logic;  -- CON730_2  front panel (DSUB HD15)
    --userio_3_p_b : out std_logic;  -- CON730_8  front panel (DSUB HD15)
    --userio_3_n_b : out std_logic;  -- CON730_3  front panel (DSUB HD15)
    --userio_4_p_b : out std_logic;  -- CON730_9  front panel (DSUB HD15)
    --userio_4_n_b : out std_logic;  -- CON730_4  front panel (DSUB HD15)
    --userio_5_p_b : out std_logic;  -- CON730_10 front panel (DSUB HD15), GCLK
    --userio_5_n_b : out std_logic;  -- CON730_5  front panel (DSUB HD15), GCLK
    userio_1_o : out std_logic;  -- CON730_6 PPS output
    userio_2_o : out std_logic;  -- CON730_1 PMU output (PPS-aligned)

    -- Front panel LEDs
    led_1_o : out std_logic;  -- LED750 (quad) front panel top
    led_2_o : out std_logic;  -- LED750 (quad) front panel
    led_3_o : out std_logic;  -- LED750 (quad) front panel
    led_4_o : out std_logic;  -- LED750 (quad) front panel bottom

    -- Internal LEDs
    --led_10_o : out std_logic;  -- LED760 (SMD), "LDB"
    --led_11_o : out std_logic;  -- LED761 (SMD), "LDA"

    -- Internal push button
    sw_button_n_i : in std_logic;  -- Push button SW770

    ---------------------------------------------------------------------------
    -- CompactRIO interface
    ---------------------------------------------------------------------------

    crio_id_select_n_i   : in  std_logic;  -- Module detect / mode select
    crio_id_select_o     : out std_logic;  -- Module present output
    crio_spi_clk_i       : in  std_logic;  -- SPI clock input
    crio_mosi_dio7_i     : in  std_logic;  -- SPI data input (optional DIO7)
    crio_miso_dio6_o     : out std_logic;  -- SPI data output (optional DIO6)
    crio_spi_cs_n_dio5_i : in  std_logic;  -- SPI CS input (optional DIO5)
    -- SPI port select if crio_id_select_n = 1, or selecting between ID mode
    -- and Auxiliary Communication mode if crio_id_select_n = 0
    crio_spi_func_dio4_i  : in  std_logic;
    crio_eeprom_cs_n_o    : out std_logic;  -- SPI CS output for ext. EEPROM
    crio_dio0_i           : in  std_logic;  -- Optional DIO
    crio_trig_out_dio1_i  : in  std_logic;  -- Optional DIO
    crio_done_n_dio2_i    : in  std_logic;  -- Optional DIO
    crio_convert_n_dio3_i : in  std_logic   -- Optional DIO
    );
end entity crio_wr_pmu_top;

architecture top of crio_wr_pmu_top is

  attribute iob : string;

  -----------------------------------------------------------------------------
  -- Constants
  -----------------------------------------------------------------------------

  -- CRIO
  constant CRIO_SPI_DATA_LENGTH : integer := 72;
  constant CRIO_SPI_CMD_LENGTH  : integer := 8;

  -- Number of masters on the wishbone crossbar
  constant c_NUM_WB_MASTERS : integer := 1;

  -- Number of slaves on the primary wishbone crossbar
  constant c_NUM_WB_SLAVES : integer := 1;

  -- Primary Wishbone master(s) offsets
  constant c_WB_MASTER_ETHBONE : integer := 0;

  -- Primary Wishbone slave(s) offsets
  constant c_WB_SLAVE_WRC : integer := 0;

  -- sdb header address on primary crossbar
  constant c_SDB_ADDRESS : t_wishbone_address := x"00040000";

  -- f_xwb_bridge_manual_sdb(size, sdb_addr)
  -- Note: sdb_addr is the sdb records address relative to the bridge base address
  constant c_wrc_bridge_sdb : t_sdb_bridge :=
    f_xwb_bridge_manual_sdb(x"0003ffff", x"00030000");

  -- Primary wishbone crossbar layout
  constant c_WB_LAYOUT : t_sdb_record_array(c_NUM_WB_SLAVES - 1 downto 0) := (
    c_WB_SLAVE_WRC => f_sdb_embed_bridge(c_wrc_bridge_sdb, x"00000000"));

  -------------------------------------------------------------------------------
  ---- Signals
  -------------------------------------------------------------------------------

  -- Wishbone buse(s) from masters attached to crossbar
  signal cnx_master_out : t_wishbone_master_out_array(c_NUM_WB_MASTERS-1 downto 0);
  signal cnx_master_in  : t_wishbone_master_in_array(c_NUM_WB_MASTERS-1 downto 0);

  -- Wishbone buse(s) to slaves attached to crossbar
  signal cnx_slave_out : t_wishbone_slave_out_array(c_NUM_WB_SLAVES-1 downto 0);
  signal cnx_slave_in  : t_wishbone_slave_in_array(c_NUM_WB_SLAVES-1 downto 0);

  -- IBUFDS
  signal clk_125m_pllref_buf : std_logic;

  -- PLLs, clocks
  signal clk_pll_62m5 : std_logic;
  signal clk_pll_125m : std_logic;
  signal clk_pll_dmtd : std_logic;
  signal pll_locked   : std_logic;
  signal clk_10m_ext  : std_logic;

  -- Reset logic
  signal rst_62m5_n       : std_logic;
  signal rst_ref_125m_n   : std_logic;
  signal rstlogic_arst_n  : std_logic;
  signal rstlogic_clk_in  : std_logic_vector(1 downto 0);
  signal rstlogic_rst_out : std_logic_vector(1 downto 0);

  -- PLL DAC ARB
  signal dac_hpll_load_p1 : std_logic;
  signal dac_hpll_data    : std_logic_vector(15 downto 0);
  signal dac_dpll_load_p1 : std_logic;
  signal dac_dpll_data    : std_logic_vector(15 downto 0);

  -- PHY
  signal phy8_to_wrc   : t_phy_8bits_to_wrc;
  signal phy8_from_wrc : t_phy_8bits_from_wrc;

  -- External reference
  signal ext_ref_mul         : std_logic;
  signal ext_ref_mul_locked  : std_logic;
  signal ext_ref_mul_stopped : std_logic;
  signal ext_ref_rst         : std_logic;

  -- I2C EEPROM
  signal eeprom_sda_in  : std_logic;
  signal eeprom_sda_out : std_logic;
  signal eeprom_scl_in  : std_logic;
  signal eeprom_scl_out : std_logic;

  -- SFP
  signal sfp_sda_in  : std_logic;
  signal sfp_sda_out : std_logic;
  signal sfp_scl_in  : std_logic;
  signal sfp_scl_out : std_logic;

  -- OneWire
  signal onewire_in : std_logic_vector(1 downto 0);
  signal onewire_en : std_logic_vector(1 downto 0);

  -- WRPC
  signal pps_csync     : std_logic;
  signal pps_pulse     : std_logic;
  signal pps_led       : std_logic;
  signal pps_led_ext   : std_logic;
  signal led_act       : std_logic;
  signal led_link      : std_logic;
  signal tm_time_valid : std_logic;
  signal tm_tai        : std_logic_vector(39 downto 0);
  signal tm_cycles     : std_logic_vector(27 downto 0);

  -- PMU
  signal pmu_pulse : std_logic;

  -- Force PPS and PMU pulses to IOBs
  attribute IOB of pps_pulse : signal is "FORCE";
  attribute IOB of pmu_pulse : signal is "FORCE";

begin  -- architecture top

  ------------------------------------------------------------------------------
  -- System reset
  ------------------------------------------------------------------------------

  -- logic AND of all async reset sources (active low)
  rstlogic_arst_n <= pll_locked and sw_button_n_i;

  -- concatenation of all clocks required to have synced resets
  rstlogic_clk_in(0) <= clk_pll_62m5;
  rstlogic_clk_in(1) <= clk_pll_125m;

  cmp_rstlogic_reset : gc_reset
    generic map (
      g_clocks    => 2,  -- 62.5MHz, 125MHz
      g_logdelay  => 4,  -- 16 clock cycles
      g_syncdepth => 3)  -- length of sync chains
    port map (
      free_clk_i => clk_125m_pllref_buf,
      locked_i   => rstlogic_arst_n,
      clks_i     => rstlogic_clk_in,
      rstn_o     => rstlogic_rst_out);

  -- distribution of resets (already synchronized to their clock domains)
  rst_62m5_n     <= rstlogic_rst_out(0);
  rst_ref_125m_n <= rstlogic_rst_out(1);

  -----------------------------------------------------------------------------
  -- Platform-dependent part (PHY, PLLs, buffers, etc)
  -----------------------------------------------------------------------------

  cmp_ibufgds_pllref : IBUFGDS
    generic map (
      DIFF_TERM    => TRUE,
      IBUF_LOW_PWR => TRUE,
      IOSTANDARD   => "DEFAULT")
    port map (
      O  => clk_125m_pllref_buf,
      I  => clk_125m_pllref_p_i,
      IB => clk_125m_pllref_n_i);

  cmp_xwrc_platform : xwrc_platform_xilinx
    generic map (
      g_fpga_family               => "spartan6",
      g_with_external_clock_input => g_with_external_clock_input,
      g_use_default_plls          => TRUE,
      g_gtp_enable_ch0            => 1,
      g_gtp_enable_ch1            => 0,
      g_simulation                => g_simulation)
    port map (
      areset_n_i            => sw_button_n_i,
      clk_10m_ext_i         => '0',
      clk_20m_vcxo_i        => clk_20m_vcxo_i,
      clk_125m_pllref_i     => clk_125m_pllref_buf,
      clk_125m_gtp_p_i      => clk_125m_gtp_p_i,
      clk_125m_gtp_n_i      => clk_125m_gtp_n_i,
      sfp_txn_o             => sfp_txn_o,
      sfp_txp_o             => sfp_txp_o,
      sfp_rxn_i             => sfp_rxn_i,
      sfp_rxp_i             => sfp_rxp_i,
      sfp_tx_fault_i        => sfp_tx_fault_i,
      sfp_los_i             => sfp_los_i,
      sfp_tx_disable_o      => sfp_tx_disable_o,
      clk_62m5_sys_o        => clk_pll_62m5,
      clk_125m_ref_o        => clk_pll_125m,
      clk_62m5_dmtd_o       => clk_pll_dmtd,
      pll_locked_o          => pll_locked,
      clk_10m_ext_o         => clk_10m_ext,
      phy8_o                => phy8_to_wrc,
      phy8_i                => phy8_from_wrc,
      ext_ref_mul_o         => ext_ref_mul,
      ext_ref_mul_locked_o  => ext_ref_mul_locked,
      ext_ref_mul_stopped_o => ext_ref_mul_stopped,
      ext_ref_rst_i         => ext_ref_rst);

  -----------------------------------------------------------------------------
  -- Primary wishbone Crossbar
  -----------------------------------------------------------------------------

  cmp_sdb_crossbar : xwb_sdb_crossbar
    generic map (
      g_num_masters => c_NUM_WB_MASTERS,
      g_num_slaves  => c_NUM_WB_SLAVES,
      g_registered  => TRUE,
      g_wraparound  => TRUE,
      g_layout      => c_WB_LAYOUT,
      g_sdb_addr    => c_SDB_ADDRESS)
    port map (
      clk_sys_i => clk_pll_62m5,
      rst_n_i   => rst_62m5_n,
      slave_i   => cnx_master_out,
      slave_o   => cnx_master_in,
      master_i  => cnx_slave_out,
      master_o  => cnx_slave_in);

  -----------------------------------------------------------------------------
  -- The WR PTP core board package (WB Slave + WB Master (Etherbone))
  -----------------------------------------------------------------------------

  cmp_board_common : xwrc_board_common
    generic map (
      g_simulation                => g_simulation,
      g_with_external_clock_input => g_with_external_clock_input,
      g_dpram_initf               => g_dpram_initf,
      g_fabric_iface              => ETHERBONE
      )
    port map (
      clk_sys_i            => clk_pll_62m5,
      clk_dmtd_i           => clk_pll_dmtd,
      clk_ref_i            => clk_pll_125m,
      clk_10m_ext_i        => clk_10m_ext,
      clk_ext_mul_i        => ext_ref_mul,
      clk_ext_mul_locked_i => ext_ref_mul_locked,
      clk_ext_stopped_i    => ext_ref_mul_stopped,
      clk_ext_rst_o        => ext_ref_rst,
      pps_ext_i            => '0',
      rst_n_i              => rst_62m5_n,
      dac_hpll_load_p1_o   => dac_hpll_load_p1,
      dac_hpll_data_o      => dac_hpll_data,
      dac_dpll_load_p1_o   => dac_dpll_load_p1,
      dac_dpll_data_o      => dac_dpll_data,
      phy8_o               => phy8_from_wrc,
      phy8_i               => phy8_to_wrc,
      scl_o                => eeprom_scl_out,
      scl_i                => eeprom_scl_in,
      sda_o                => eeprom_sda_out,
      sda_i                => eeprom_sda_in,
      sfp_scl_o            => sfp_scl_out,
      sfp_scl_i            => sfp_scl_in,
      sfp_sda_o            => sfp_sda_out,
      sfp_sda_i            => sfp_sda_in,
      sfp_det_i            => sfp_mod_def0_i,
      spi_sclk_o           => flash_sclk_o,
      spi_ncs_o            => flash_ncs_o,
      spi_mosi_o           => flash_mosi_o,
      spi_miso_i           => flash_miso_i,
      uart_rxd_i           => uart_rxd_i,
      uart_txd_o           => uart_txd_o,
      owr_en_o             => onewire_en,
      owr_i                => onewire_in,
      wb_slave_i           => cnx_slave_in(c_WB_SLAVE_WRC),
      wb_slave_o           => cnx_slave_out(c_WB_SLAVE_WRC),
      wb_eth_master_o      => cnx_master_out(c_WB_MASTER_ETHBONE),
      wb_eth_master_i      => cnx_master_in(c_WB_MASTER_ETHBONE),
      tm_time_valid_o      => tm_time_valid,
      tm_tai_o             => tm_tai,
      tm_cycles_o          => tm_cycles,
      led_act_o            => led_act,
      led_link_o           => led_link,
      pps_csync_o          => pps_csync,
      pps_p_o              => pps_pulse,
      pps_led_o            => pps_led);

  sfp_rate_select_o <= '1';

  -- Tristates
  sfp_mod_def1_b <= '0' when sfp_scl_out = '0' else 'Z';
  sfp_mod_def2_b <= '0' when sfp_sda_out = '0' else 'Z';
  sfp_scl_in     <= sfp_mod_def1_b;
  sfp_sda_in     <= sfp_mod_def2_b;

  onewire_b     <= '0' when (onewire_en(0) = '1') else 'Z';
  onewire_in(0) <= onewire_b;
  onewire_in(1) <= '1';

  eeprom_sda_b  <= '0' when (eeprom_sda_out = '0') else 'Z';
  eeprom_sda_in <= eeprom_sda_b;
  eeprom_scl_b  <= '0' when (eeprom_scl_out = '0') else 'Z';
  eeprom_scl_in <= eeprom_scl_b;

  ------------------------------------------------------------------------------
  -- CRIO SPI DUAL DAC
  ------------------------------------------------------------------------------
  cmp_dac_arb : entity work.cutewr_serial_dac_arb
    generic map (
      g_invert_sclk    => FALSE,
      g_num_extra_bits => 8
      )
    port map (
      clk_i   => clk_pll_62m5,
      rst_n_i => rst_62m5_n,

      val1_i  => dac_dpll_data,
      load1_i => dac_dpll_load_p1,

      val2_i  => dac_hpll_data,
      load2_i => dac_hpll_load_p1,

      dac_sync_n_o => plldac_sync_n_o,
      dac_ldac_n_o => plldac_ldac_n_o,
      dac_clr_n_o  => plldac_clr_n_o,
      dac_sclk_o   => plldac_sclk_o,
      dac_din_o    => plldac_din_o
      );

  ------------------------------------------------------------------------------
  -- CRIO interface
  ------------------------------------------------------------------------------

  cmp_crio_arb : entity work.crio_arb
    generic map(
      SPI_DATA_LENGTH => CRIO_SPI_DATA_LENGTH,
      SPI_CMD_LENGTH  => CRIO_SPI_CMD_LENGTH
      )
    port map(
      clk_i   => clk_pll_125m,
      rst_n_i => rst_ref_125m_n,

      crio_spi_clk_i  => crio_spi_clk_i,
      crio_spi_cs_n_i => crio_spi_cs_n_dio5_i,
      crio_spi_mosi_i => crio_mosi_dio7_i,
      crio_spi_miso_o => crio_miso_dio6_o,

      crio_id_select_n_i      => crio_id_select_n_i,
      crio_id_select_o        => crio_id_select_o,
      crio_id_select_o_oe_n_o => open,

      crio_spi_func_i => crio_spi_func_dio4_i,

      crio_spi_eeprom_cs_n_o => crio_eeprom_cs_n_o,

      crio_spi_eeprom_cs_led_n_o => open,
      crio_spi_data_cs_led_n_o   => open,

      wrpc_status_time_i => "00" & led_link & tm_time_valid & tm_tai & tm_cycles,

      data_reg_scratch1_o => open,
      data_reg_scratch2_o => open
      );

  -----------------------------------------------------------------------------
  -- PMU pulse generation
  -- for now this is 50Hz, 50% duty cycle, in the future we can make
  -- it configurable.
  -- A better implementation could be to borrow the 10MHz
  -- clock generation module from the WR Switch project, which makes use
  -- of an OSERDES and ODELAY block (to allow fine alignment of the PMU pulse
  -- to the actual PPS).
  -- Don't forget to force the placement of the output to an IOB, to get
  -- consistent timing.
  -----------------------------------------------------------------------------
  p_pmu_pulse_gen : process (clk_pll_125m) is
    variable v_pmu_cnt : unsigned(21 downto 0);

  begin  -- process p_pmu_pulse_gen
    if rising_edge(clk_pll_125m) then
      -- in case of reset, pps_csync or PMU pulse period
      -- reset the counter, otherwise inc by 1
      if (rst_ref_125m_n = '0') or (pps_csync = '1') or
        (v_pmu_cnt = to_unsigned(2500000, v_pmu_cnt'length)) then
        v_pmu_cnt := to_unsigned(1, v_pmu_cnt'length);
      else
        v_pmu_cnt := v_pmu_cnt + 1;
      end if;
      -- drive the PMU pulse based on the counter value
      if v_pmu_cnt > to_unsigned(1250000, v_pmu_cnt'length) then
        pmu_pulse <= '0';
      else
        pmu_pulse <= '1';
      end if;
    end if;
  end process p_pmu_pulse_gen;

------------------------------------------------------------------------------
-- Front panel
------------------------------------------------------------------------------

  cmp_extend_pps : gc_extend_pulse
    generic map (
      g_width => 10000000
      )
    port map (
      clk_i      => clk_pll_125m,
      rst_n_i    => rst_ref_125m_n,
      pulse_i    => pps_led,
      extended_o => pps_led_ext
      );

  led_1_o <= not led_act;        -- LED is active L, at the top
  led_2_o <= not led_link;       -- LED is active L
  led_3_o <= not tm_time_valid;  -- LED is active L
  led_4_o <= not pps_led_ext;    -- LED is active L, at the bottom

  userio_1_o <= pps_pulse;
  userio_2_o <= pmu_pulse;

end architecture top;
